-----------------------------------------------------------------------------------------------
-- Client Lua Script for ChatSounds
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------

require "Window"
require "Apollo"
require "ApolloColor"
require "Sound"
require "GameLib"
require "ChatSystemLib"

-----------------------------------------------------------------------------------------------
-- ChatSounds Module Definition
-----------------------------------------------------------------------------------------------
local ChatSounds = {}

-----------------------------------------------------------------------------------------------
-- Upvalues
-----------------------------------------------------------------------------------------------
local type = type
local next = next
local unpack = unpack
local strfind = string.find
local strlower = string.lower
local strformat = string.format
local strsplit = function(strInput)
  local t={}
  for word in string.gmatch(strInput,"%S+") do
    table.insert(t,word)
  end
  if next(t) then
    return unpack(t)
  end
end
local tonumber = tonumber
local CarbineChatLog
local CarbineFloatText

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local strMyName, strFirst, strLast
--[[

Suitable game sounds: blips and ticks used mostly for interface functions so they\'re not confusing

	PlayUI22And25GenericToggleOnPhysical02

	PlayUI43RotateRightClickDigital

	PlayUI43RotateLeftClickDigital

	PlayUI47CancelVirtual

	PlayUI53SlideUpCancelVirtual

	PlayUI54SlideDownDigital

	PlayUI55ErrorVirtual

	PlayUI60MessageAlertDigital

	PlayUIAlertPopUpMessageReceived

	PlayUIAnimPulseHolo

	PlayUIButtonHoloLarge

	PlayUIButtonMetalSmall

	PlayUICraftingCoordinateHit

	PlayUIDatachronBeep

	PlayUIWindowAccordionClose

	PlayUIWindowAccordionOpen

	PlayUIWindowHoloOpen

	PlayUIWIndowMetalClose

	PlayUIWindowMetalOpen

	PlayUISocialWhisper

]]
local tkMyGameSounds = {
  ["GM"] = Sound.PlayUI60MessageAlertDigital,
  ["MyName"] = Sound.PlayUI55ErrorVirtual,
  ["ChatChannel_Party"] = Sound.PlayUIButtonHoloLarge,
  ["ChatChannel_Instance"] = Sound.PlayUIButtonHoloLarge,
  ["ChatChannel_Guild"] = Sound.PlayUIWIndowMetalClose,
  ["ChatChannel_GuildOfficer"] = Sound.PlayUIWindowMetalOpen,
  ["ChatChannel_WarParty"] = Sound.PlayUIWindowAccordionClose,
  ["ChatChannel_WarPartyOfficer"] = Sound.PlayUIAlertPopUpMessageReceived,
  ["ChatChannel_Society"] = Sound.PlayUIWindowHoloOpen,
}
local tkMyExternalSounds = {

  }
local tkChannelsMyName = {
  [ChatSystemLib.ChatChannel_Say] 			= true,
  [ChatSystemLib.ChatChannel_Yell] 			= true,
  [ChatSystemLib.ChatChannel_Zone] 			= true,
  [ChatSystemLib.ChatChannel_ZonePvP] 		= true,
  [ChatSystemLib.ChatChannel_Trade] 			= true,
  [ChatSystemLib.ChatChannel_Custom] 			= true,
}
local tkChannelsWatched = {
  [ChatSystemLib.ChatChannel_Party] 			= true,
  [ChatSystemLib.ChatChannel_Guild] 			= true,
  [ChatSystemLib.ChatChannel_GuildOfficer] 	= true,
  [ChatSystemLib.ChatChannel_Society] 		= true,
  [ChatSystemLib.ChatChannel_Instance] 		= true,
  [ChatSystemLib.ChatChannel_WarParty] 		= true,
  [ChatSystemLib.ChatChannel_WarPartyOfficer] = true,
}
local tkSoundMap = {
  [ChatSystemLib.ChatChannel_Party] 			= tkMyGameSounds.ChatChannel_Party,
  [ChatSystemLib.ChatChannel_Guild] 			= tkMyGameSounds.ChatChannel_Guild,
  [ChatSystemLib.ChatChannel_GuildOfficer] 	= tkMyGameSounds.ChatChannel_GuildOfficer,
  [ChatSystemLib.ChatChannel_Society] 		= tkMyGameSounds.ChatChannel_Society,
  [ChatSystemLib.ChatChannel_Instance] 		= tkMyGameSounds.ChatChannel_Instance,
  [ChatSystemLib.ChatChannel_WarParty] 		= tkMyGameSounds.ChatChannel_WarParty,
  [ChatSystemLib.ChatChannel_WarPartyOfficer] = tkMyGameSounds.ChatChannel_WarPartyOfficer,
}
for k in pairs(tkChannelsMyName) do
  tkSoundMap[k] = tkMyGameSounds.MyName
end
local tkVisibleChannels = {}
local bStatusEnabled = true
local Events = {}
local tTextOption
local arChatColor

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ChatSounds:new(o)
  o = o or {}
  setmetatable(o, self)
  self.__index = self
  -- initialize variables here

  return o
end

function ChatSounds:Init()
  local bHasConfigureFunction = false
  local strConfigureButtonText = ""
  local tDependencies = {
    -- "UnitOrPackageName",
    }
  Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end


-----------------------------------------------------------------------------------------------
-- ChatSounds OnLoad
-----------------------------------------------------------------------------------------------
function ChatSounds:OnLoad()
  -- load our form file
  self.xmlDoc = XmlDoc.CreateFromFile("ChatSounds.xml")
  self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- ChatSounds OnDocLoaded
-----------------------------------------------------------------------------------------------
function ChatSounds:OnDocLoaded()

  if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
    self.wndMain = Apollo.LoadForm(self.xmlDoc, "ChatSoundsForm", nil, self)
    if self.wndMain == nil then
      Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
      return
    end

    self.wndMain:Show(false, true)
    self.wndChannelSettings = self.wndMain:FindChild("ChannelSettings")

    self.wndParty = self.wndChannelSettings:FindChild("Party")
    self.wndPartyContainer = self.wndParty:FindChild("ChoiceContainer")

    self.wndInstance = self.wndChannelSettings:FindChild("Instance")
    self.wndInstanceContainer = self.wndInstance:FindChild("ChoiceContainer")

    self.wndWarParty = self.wndChannelSettings:FindChild("WarParty")
    self.wndWarPartyContainer = self.wndWarParty:FindChild("ChoiceContainer")

    self.wndWarPartyOfficer = self.wndChannelSettings:FindChild("WarPartyOfficer")
    self.wndWarPartyOfficerContainer = self.wndWarPartyOfficer:FindChild("ChoiceContainer")

    self.wndGuild = self.wndChannelSettings:FindChild("Guild")
    self.wndGuildContainer = self.wndGuild:FindChild("ChoiceContainer")

    self.wndGuildOfficer = self.wndChannelSettings:FindChild("GuildOfficer")
    self.wndGuildOfficerContainer = self.wndGuildOfficer:FindChild("ChoiceContainer")

    self.wndSociety = self.wndChannelSettings:FindChild("Society")
    self.wndSocietyContainer = self.wndSociety:FindChild("ChoiceContainer")

    -- Register handlers for events, slash commands and timer, etc.
    -- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
    Apollo.RegisterSlashCommand("chatsounds", "OnChatSoundsOn", self)

    -- delay init
    self.timer = ApolloTimer.Create(2.0, false, "DelayInit", self)

    -- Do additional Addon initialization here
  end
end

-----------------------------------------------------------------------------------------------
-- ChatSounds Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/chatsounds"
function ChatSounds:OnChatSoundsOn()
  -- options not implemented yet TODO: Dropdowns to select among available sounds or none, preview sound on select.
  self.wndMain:Invoke() -- show the window
  -- let\'s give them an on/off toggle for now
  bStatusEnabled = not bStatusEnabled
  if bStatusEnabled and not Events.ChatMessage then
    Apollo.RegisterEventHandler("ChatMessage", "OnChatMessage", self)
    Events.ChatMessage = true
  elseif not bStatusEnabled and Events.ChatMessage then
    Apollo.RemoveEventHandler("ChatMessage",self)
    Events.ChatMessage = nil
  end
  ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System,"ChatSounds: "..(bStatusEnabled and "ON" or "OFF"))
end

function ChatSounds:DelayInit()
  strMyName = GameLib.GetPlayerUnit():GetName()
  strFirst, strLast = strsplit(strMyName)
  -- might use first or last name matches if I can think of a good way to weed out all the "My" "Mr" "A" false positives.
  CarbineChatLog = Apollo.GetAddon('ChatLog')
  if CarbineChatLog then
    arChatColor = TableUtil:Copy(CarbineChatLog.arChatColor)
  end
  CarbineFloatText = Apollo.GetAddon('FloatText') or Apollo.GetAddon('CombatText')
  if CarbineFloatText then
    tTextOption = CarbineFloatText:GetDefaultTextOption()
  end
  -- Print(Apollo.GetAssetFolder())
	self:UpdateVisibleChannels()
	
  Apollo.RegisterEventHandler("ChatMessage", "OnChatMessage", self)
  Apollo.RegisterEventHandler("ChatJoin", "UpdateVisibleChannels", self)
  Apollo.RegisterEventHandler("ChatLeave", "UpdateVisibleChannels", self)
  Events.ChatMessage = true
  ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System,"ChatSounds Loaded. /chatsounds")
end

function ChatSounds:FloatText(strText,eChannelType)
  if tTextOption then
    tTextOption.strFontFace = "CRB_FloaterSmall"
    tTextOption.bUseScreenPos = true
    tTextOption.fOffset = -250
    tTextOption.nColor = self:ApolloColorToHEX(arChatColor[eChannelType] or ApolloColor.new("green"))
    tTextOption.bShowOnTop = true
    CarbineFloatText:RequestShowTextFloater(LuaEnumMessageType.SystemMessage, GameLib.GetControlledUnit(), strText, tTextOption, 0)
  end
end

-----------------------------------------------------------------------------------------------
-- ChatSoundsForm Functions
-----------------------------------------------------------------------------------------------
function ChatSounds:OptionsShow( wndHandler, wndControl )
  self.wndParty:FindChild("DropPartySound"):SetText("PlayUIButtonHoloLarge")
  self.wndPartyContainer:Show(false, true)
  self.wndInstanceContainer:Show(false, true)
  self.wndWarPartyContainer:Show(false, true)
  self.wndWarPartyOfficerContainer:Show(false, true)
  self.wndGuildContainer:Show(false, true)
  self.wndGuildOfficerContainer:Show(false, true)
  self.wndSocietyContainer:Show(false, true)
end

function ChatSounds:OnOK()
  self.wndMain:Close() -- hide the window
end
function ChatSounds:OnCancel()
  self.wndMain:Close() -- hide the window
end

function ChatSounds:Enable( wndHandler, wndControl, eMouseButton )
end

function ChatSounds:Disable( wndHandler, wndControl, eMouseButton )
end

function ChatSounds:OnSelectChannelSound( wndHandler, wndControl, eMouseButton )
end

function ChatSounds:OnPartyToggle( wndHandler, wndControl, eMouseButton )
end

function ChatSounds:OnInstanceToggle( wndHandler, wndControl, eMouseButton )
end

function ChatSounds:OnWarPartyToggle( wndHandler, wndControl, eMouseButton )
end

function ChatSounds:OnWarPartyOfficerToggle( wndHandler, wndControl, eMouseButton )
end

function ChatSounds:OnGuildToggle( wndHandler, wndControl, eMouseButton )
end

function ChatSounds:OnGuildOfficerToggle( wndHandler, wndControl, eMouseButton )
end

function ChatSounds:OnSocietyToggle( wndHandler, wndControl, eMouseButton )
end

-----------------------------------------------------------------------------------------------
-- Events
-----------------------------------------------------------------------------------------------
function ChatSounds:OnChatMessage(channelCurrent, tMessage)
  -- tMessage has bAutoResponse, bGM, bSelf, strSender, strRealmName, nPresenceState, arMessageSegments, unitSource, bShowChatBubble, bCrossFaction, nReportId

  -- arMessageSegments is an array of tables.  Each table represents a part of the message + the formatting for that segment.
  -- This allows us to signal font (alien text for example) changes mid stream.
  -- local example = arMessageSegments[1]
  -- example.strText is the text
  -- example.bAlien == true if alien font set
  -- example.bRolePlay == true if this is rolePlay Text.  RolePlay text should only show up for people in roleplay mode, and non roleplay text should only show up for people outside it.

  -- to use: 	{#}toggles alien on {*}toggles rp on. Alien is still on {!}resets all format codes.
  local eChannelType,strChannelName,strUniqueId
  eChannelType = channelCurrent:GetType()
  strChannelName = channelCurrent:GetName()
  strUniqueId = channelCurrent:GetUniqueId()
  if not eChannelType or not (tkChannelsWatched[eChannelType] or tkChannelsMyName[eChannelType]) then return end
  -- we only care for incoming
  if tMessage.bSelf then return end -- comment out for debugging, use Print("blah") in here to avoid infinite event loops
  -- if we have a channel filter from Carbine's ChatLog don't alert for filtered channels (TODO: does the event fire for filtered messages?)
  if next(tkVisibleChannels) and not (tkVisibleChannels[eChannelType] or tkVisibleChannels[strUniqueId]) then return end
  -- always alert if we get a message from GM (TODO: check if the game already has a sound alert)
  if tMessage.bGM then Sound.Play(tkMyGameSounds.GM) return end
  -- only alert on global channels if our character name is mentioned TODO: support alias/nick/alt
  if tkChannelsMyName[eChannelType] then
    for _,v in ipairs(tMessage.arMessageSegments) do
      if strfind(strlower(v.strText),strlower(strMyName)) then
        Sound.Play(tkSoundMap[eChannelType])
        self:FloatText(v.strText,eChannelType)
        break
      end
    end
  else
    Sound.Play(tkSoundMap[eChannelType])
  end
end
-----------------------------------------------------------------------------------------------
-- Utilities
-----------------------------------------------------------------------------------------------
function ChatSounds:TableWipe(t)
  if t==nil then return {} end
  if not type(t)=="table" then return end
  for k in pairs(t) do
    t[k]=nil
  end
  return t
end
local tColor = {}
function ChatSounds:ApolloColorToHEX(apoloColor)
  if not apoloColor or not type(apoloColor)=="userdata" then return 0xcc0000 end
  tColor = self:TableWipe(tColor)
  tColor = TableUtil:Copy(apoloColor:ToTable())
  -- return strformat("%02x%02x%02x%02x", tColor.a * 255, tColor.r * 255, tColor.g * 255, tColor.b * 255)
  return tonumber(strformat("0x%02x%02x%02x", tColor.r * 255, tColor.g * 255, tColor.b * 255),16)
end
function ChatSounds:UpdateVisibleChannels()
	  if CarbineChatLog and CarbineChatLog.tChatWindows then
    tkVisibleChannels = self:TableWipe(tkVisibleChannels)
    for _,wndChat in pairs(CarbineChatLog.tChatWindows) do
      for k,v in pairs(wndChat:GetData().tViewedChannels) do
        if not tkVisibleChannels[k] and v then
          tkVisibleChannels[k] = v
        end
      end
    end
  end
end
-----------------------------------------------------------------------------------------------
-- ChatSounds Instance
-----------------------------------------------------------------------------------------------
local ChatSoundsInst = ChatSounds:new()
ChatSoundsInst:Init()
